/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workitem.web;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workitem.entity.WorkItem;
import com.jeeplus.modules.workitem.service.WorkItemService;

/**
 * 物品一览表Controller
 * @author chenyuping
 * @version 2018-07-13
 */
@Controller
@RequestMapping(value = "${adminPath}/workitem/workItem")
public class WorkItemController extends BaseController {

	@Autowired
	private WorkItemService workItemService;
	
	@ModelAttribute
	public WorkItem get(@RequestParam(required=false) String id) {
		WorkItem entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workItemService.get(id);
		}
		if (entity == null){
			entity = new WorkItem();
		}
		return entity;
	}
	
	/**
	 * 物品一览表列表页面
	 */
	@RequiresPermissions("workitem:workItem:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkItem workItem, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkItem> page = workItemService.findPage(new Page<WorkItem>(request, response), workItem); 
		model.addAttribute("page", page);
		return "modules/workitem/workItemList";
	}

	/**
	 * 查看，增加，编辑物品一览表表单页面
	 */
	@RequiresPermissions(value={"workitem:workItem:view","workitem:workItem:add","workitem:workItem:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkItem workItem, Model model) {
		model.addAttribute("workItem", workItem);
		return "modules/workitem/workItemForm";
	}

	/**
	 * 保存物品一览表
	 */
	@RequiresPermissions(value={"workitem:workItem:add","workitem:workItem:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkItem workItem, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workItem)){
			return form(workItem, model);
		}
		if(!workItem.getIsNewRecord()){//编辑表单保存
			WorkItem t = workItemService.get(workItem.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workItem, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workItemService.save(t);//保存
		}else{//新增表单保存
			workItemService.save(workItem);//保存
		}
		addMessage(redirectAttributes, "保存物品一览表成功");
		return "redirect:"+Global.getAdminPath()+"/workitem/workItem/?repage";
	}
	
	/**
	 * 删除物品一览表
	 */
	@RequiresPermissions("workitem:workItem:del")
	@RequestMapping(value = "delete")
	public String delete(WorkItem workItem, RedirectAttributes redirectAttributes) {
		workItemService.delete(workItem);
		addMessage(redirectAttributes, "删除物品一览表成功");
		return "redirect:"+Global.getAdminPath()+"/workitem/workItem/?repage";
	}
	
	/**
	 * 批量删除物品一览表
	 */
	@RequiresPermissions("workitem:workItem:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workItemService.delete(workItemService.get(id));
		}
		addMessage(redirectAttributes, "删除物品一览表成功");
		return "redirect:"+Global.getAdminPath()+"/workitem/workItem/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workitem:workItem:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkItem workItem, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "物品一览表"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkItem> page = workItemService.findPage(new Page<WorkItem>(request, response, -1), workItem);
    		new ExportExcel("物品一览表", WorkItem.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出物品一览表记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitem/workItem/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workitem:workItem:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkItem> list = ei.getDataList(WorkItem.class);
			for (WorkItem workItem : list){
				try{
					workItemService.save(workItem);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条物品一览表记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条物品一览表记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入物品一览表失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitem/workItem/?repage";
    }
	
	/**
	 * 下载导入物品一览表数据模板
	 */
	@RequiresPermissions("workitem:workItem:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "物品一览表数据导入模板.xlsx";
    		List<WorkItem> list = Lists.newArrayList(); 
    		new ExportExcel("物品一览表数据", WorkItem.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitem/workItem/?repage";
    }


	/**
	 * 选择固定资产
	 */
	@RequestMapping(value = "selectItem")
	public String selectItem(WorkItem workItem, String url, String fieldLabels, String fieldKeys, String searchLabel, String searchKey,String isOne, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkItem> page = workItemService.findItemPage(new Page<WorkItem>(request, response),  workItem);
		try {
			fieldLabels = URLDecoder.decode(fieldLabels, "UTF-8");
			fieldKeys = URLDecoder.decode(fieldKeys, "UTF-8");
			searchLabel = URLDecoder.decode(searchLabel, "UTF-8");
			searchKey = URLDecoder.decode(searchKey, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		model.addAttribute("labelNames", fieldLabels.split("\\|"));
		model.addAttribute("labelValues", fieldKeys.split("\\|"));
		model.addAttribute("fieldLabels", fieldLabels);
		model.addAttribute("fieldKeys", fieldKeys);
		model.addAttribute("url", url);
		model.addAttribute("searchLabel", searchLabel);
		model.addAttribute("searchKey", searchKey);
		model.addAttribute("obj", workItem);
		model.addAttribute("page", page);
		if("true".equals(isOne)){
			return "modules/sys/gridselectOneItem";
		}else{
			return "modules/sys/gridselectItem";
		}

	}
}