/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workitemstock.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workitemstock.entity.WorkItemStock;
import com.jeeplus.modules.workitemstock.service.WorkItemStockService;

/**
 * 采购验收物品Controller
 * @author lw
 * @version 2018-07-18
 */
@Controller
@RequestMapping(value = "${adminPath}/workitemstock/workItemStock")
public class WorkItemStockController extends BaseController {

	@Autowired
	private WorkItemStockService workItemStockService;
	
	@ModelAttribute
	public WorkItemStock get(@RequestParam(required=false) String id) {
		WorkItemStock entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workItemStockService.get(id);
		}
		if (entity == null){
			entity = new WorkItemStock();
		}
		return entity;
	}
	
	/**
	 * 采购验收物品列表页面
	 */
	@RequiresPermissions("workitemstock:workItemStock:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkItemStock workItemStock, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkItemStock> page = workItemStockService.findPage(new Page<WorkItemStock>(request, response), workItemStock); 
		model.addAttribute("page", page);
		return "modules/workitemstock/workItemStockList";
	}

	/**
	 * 查看，增加，编辑采购验收物品表单页面
	 */
	@RequiresPermissions(value={"workitemstock:workItemStock:view","workitemstock:workItemStock:add","workitemstock:workItemStock:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkItemStock workItemStock, Model model) {
		model.addAttribute("workItemStock", workItemStock);
		return "modules/workitemstock/workItemStockForm";
	}

	/**
	 * 保存采购验收物品
	 */
	@RequiresPermissions(value={"workitemstock:workItemStock:add","workitemstock:workItemStock:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkItemStock workItemStock, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workItemStock)){
			return form(workItemStock, model);
		}
		if(!workItemStock.getIsNewRecord()){//编辑表单保存
			WorkItemStock t = workItemStockService.get(workItemStock.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workItemStock, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workItemStockService.save(t);//保存
		}else{//新增表单保存
			workItemStockService.save(workItemStock);//保存
		}
		addMessage(redirectAttributes, "保存采购验收物品成功");
		return "redirect:"+Global.getAdminPath()+"/workitemstock/workItemStock/?repage";
	}
	
	/**
	 * 删除采购验收物品
	 */
	@RequiresPermissions("workitemstock:workItemStock:del")
	@RequestMapping(value = "delete")
	public String delete(WorkItemStock workItemStock, RedirectAttributes redirectAttributes) {
		workItemStockService.delete(workItemStock);
		addMessage(redirectAttributes, "删除采购验收物品成功");
		return "redirect:"+Global.getAdminPath()+"/workitemstock/workItemStock/?repage";
	}
	
	/**
	 * 批量删除采购验收物品
	 */
	@RequiresPermissions("workitemstock:workItemStock:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workItemStockService.delete(workItemStockService.get(id));
		}
		addMessage(redirectAttributes, "删除采购验收物品成功");
		return "redirect:"+Global.getAdminPath()+"/workitemstock/workItemStock/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workitemstock:workItemStock:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkItemStock workItemStock, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "采购验收物品"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkItemStock> page = workItemStockService.findPage(new Page<WorkItemStock>(request, response, -1), workItemStock);
    		new ExportExcel("采购验收物品", WorkItemStock.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出采购验收物品记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitemstock/workItemStock/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workitemstock:workItemStock:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkItemStock> list = ei.getDataList(WorkItemStock.class);
			for (WorkItemStock workItemStock : list){
				try{
					workItemStockService.save(workItemStock);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条采购验收物品记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条采购验收物品记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入采购验收物品失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitemstock/workItemStock/?repage";
    }
	
	/**
	 * 下载导入采购验收物品数据模板
	 */
	@RequiresPermissions("workitemstock:workItemStock:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "采购验收物品数据导入模板.xlsx";
    		List<WorkItemStock> list = Lists.newArrayList(); 
    		new ExportExcel("采购验收物品数据", WorkItemStock.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitemstock/workItemStock/?repage";
    }
	
	
	

}